"""Distances tests module.

Copyright (c) 2020 Thierry P.G. DECKER
All Rights Reserved.
Released under the MIT license

"""
import pytest

from siarnaq.distances import Distance


def test_instanciations():
    assert isinstance(Distance(), Distance)

    assert isinstance(Distance('km'), Distance)
    assert isinstance(Distance(scale='km'), Distance)
    assert isinstance(Distance('mi'), Distance)
    assert isinstance(Distance(scale='mi'), Distance)

    with pytest.raises(Exception):
        assert Distance(scale='Dummy')
    with pytest.raises(Exception):
        assert Distance(dist='Dummy')
    with pytest.raises(Exception):
        assert Distance(scale='Dummy', dist='Dummy')


def test_conv_km_to_mi():
    assert round(Distance.conv_km_to_mi(1), 2) == 0.62


def test_conv_mi_to_km():
    assert round(Distance.conv_mi_to_km(1), 2) == 1.61


def test_propertty_getter_scales():
    assert Distance().scales == {'km', 'mi'}


def test_propertty_getter_scale():
    assert Distance().scale == 'km'
    assert Distance(scale='km').scale == 'km'
    assert Distance(scale='mi').scale == 'mi'


def test_property_getter_dist():
    assert Distance(dist=10).dist == 10
    assert Distance(scale='km', dist=10).dist == 10
    assert Distance(scale='mi', dist=10).dist == 10


def test_property_getter_kilometer():
    assert Distance(scale='km', dist=10).kilometer == 10
    assert Distance(scale='mi', dist=10).kilometer == 16.09


def test_property_getter_mile():
    assert round(Distance(scale='km', dist=10).mile, 2) == 6.22
    assert Distance(scale='mi', dist=10).mile == 10


def test_property_setter_dist():
    r = Distance(scale='km')
    r.dist = 10
    assert r.dist == 10


def test_property_setter_scale():
    with pytest.raises(Exception):
        r = Distance(scale='km', dist=1)
        r.scale = 'Dummy'
    r = Distance(scale='km', dist=1)
    r.scale = 'mi'
    assert r.scale == 'mi'
    assert round(r.dist, 2) == 0.62
    assert round(r.mile, 2) == 0.62
    assert round(r.kilometer, 2) == 1
    r = Distance(scale='mi', dist=1)
    r.scale = 'km'
    assert r.scale == 'km'
    assert round(r.dist, 2) == 1.61
    assert round(r.mile, 2) == 1
    assert round(r.kilometer, 2) == 1.61


def test_str():
    assert str(Distance()) == '0.0 km'
    assert str(Distance(scale='mi', dist=1)) == '1.0 mi'


def test_repr():
    assert repr(Distance()) == 'Distance(\'km\', 0.0)'
    assert repr(Distance(scale='mi', dist=1)) == 'Distance(\'mi\', 1.0)'


def test_add():
    r = Distance(dist=10)
    r = r + 1
    assert r.dist == 11
    r += 1
    assert r.dist == 12
    r = 1 + r
    assert r.dist == 13

    r1 = Distance(dist=10)
    r2 = Distance(dist=10)
    r = r1 + r2
    assert r.scale == 'km'
    assert r.dist == 20
    assert r.kilometer == 20
    assert round(r.mile, 2) == 12.43

    r1 = Distance(scale='mi', dist=10)
    r2 = Distance(scale='mi', dist=10)
    r1 += r2
    assert r1.scale == 'mi'
    assert r1.dist == 20
    assert round(r1.kilometer, 2) == 32.18
    assert r1.mile == 20

    r1 = Distance(scale='mi', dist=10)
    r2 = Distance(scale='km', dist=10)
    r1 += r2
    assert r1.scale == 'mi'
    assert round(r1.dist, 2) == 16.22
    assert round(r1.kilometer, 2) == 26.09
    assert round(r1.mile, 2) == 16.22


def test_sub():
    r = Distance(dist=10)
    r = r - 1
    assert r.dist == 9
    r -= 1
    assert r.dist == 8

    r1 = Distance(scale='km', dist=5)
    r2 = Distance(scale='km', dist=5)
    r = r1 - r2
    assert r.scale == 'km'
    assert round(r.dist, 2) == 0.00

    r1 = Distance(scale='km', dist=5)
    r2 = Distance(scale='km', dist=5)
    r1 -= r2
    assert r.scale == 'km'
    assert round(r.dist, 2) == 0.00

    r1 = Distance(scale='mi', dist=5)
    r2 = Distance(scale='km', dist=5)
    r = r1 - r2
    assert r.scale == 'mi'
    assert round(r.dist, 2) == 1.89

    with pytest.raises(Exception):
        r = 1 - r


def test_mul():
    r = Distance(scale='mi', dist=10)
    r = r * 2
    assert r.dist == 20
    assert r.scale == 'mi'
    r *= 2
    assert r.dist == 40
    assert r.scale == 'mi'
    r = 2 * r
    assert r.dist == 80
    assert r.scale == 'mi'


def test_div():
    r = Distance(scale='km', dist=100)
    r = r / 2
    assert r.dist == 50
    assert r.scale == 'km'
    r /= 2
    assert r.dist == 25
    assert r.scale == 'km'
    with pytest.raises(Exception):
        r = 2 / r



"""Degrees tests module.

Copyright (c) 2020 Thierry P.G. DECKER
All Rights Reserved.
Released under the MIT license

"""

import pytest

from siarnaq.degrees import Degree


def test_instanciations():
    assert isinstance(Degree(), Degree)

    assert isinstance(Degree('ce'), Degree)
    assert isinstance(Degree('fa'), Degree)
    assert isinstance(Degree('ke'), Degree)
    assert isinstance(Degree('ra'), Degree)

    assert isinstance(Degree(temp=10), Degree)

    assert isinstance(Degree(scale='ce'), Degree)
    assert isinstance(Degree(scale='fa'), Degree)
    assert isinstance(Degree(scale='ke'), Degree)
    assert isinstance(Degree(scale='ra'), Degree)

    assert isinstance(Degree(scale='ce', temp=0), Degree)
    assert isinstance(Degree(scale='fa', temp=0), Degree)
    assert isinstance(Degree(scale='ke', temp=0), Degree)
    assert isinstance(Degree(scale='ra', temp=0), Degree)

    with pytest.raises(Exception):
        assert Degree(scale='Dummy')
    with pytest.raises(Exception):
        assert Degree(temp='Dummy')
    with pytest.raises(Exception):
        assert Degree(scale='Dummy', temp='Dummy')


def test_static_methods():
    assert round(Degree.conv_ce_to_fa(temp=0), 2) == 32.00
    assert round(Degree.conv_ce_to_ke(temp=0), 2) == 273.15
    assert round(Degree.conv_ce_to_ra(temp=0), 2) == 491.67
    assert round(Degree.conv_fa_to_ce(temp=32), 2) == 0.00
    assert round(Degree.conv_fa_to_ke(temp=32), 2) == 273.15
    assert round(Degree.conv_fa_to_ra(temp=32), 2) == 491.67
    assert round(Degree.conv_ke_to_ce(temp=0), 2) == -273.15
    assert round(Degree.conv_ke_to_fa(temp=273.15), 2) == 32.00
    assert round(Degree.conv_ke_to_ra(temp=100), 2) == 180
    assert round(Degree.conv_ra_to_ce(temp=100), 2) == -217.59
    assert round(Degree.conv_ra_to_fa(temp=100), 2) == -359.67
    assert round(Degree.conv_ra_to_ke(temp=100), 2) == 55.56


def test_propertties_getters():
    assert Degree().scales == {'ce', 'fa', 'ke', 'ra'}

    r = Degree(scale='ce', temp=0)
    assert r.scale == 'ce'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == 0.00
    assert round(r.fahrenheit, 2) == 32.00
    assert round(r.kelvin, 2) == 273.15
    assert round(r.rankine, 2) == 491.67

    r = Degree(scale='fa', temp=0)
    assert r.scale == 'fa'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == -17.78
    assert round(r.fahrenheit, 2) == 0
    assert round(r.kelvin, 2) == 255.37
    assert round(r.rankine, 2) == 459.67

    r = Degree(scale='ke', temp=0)
    assert r.scale == 'ke'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0
    assert round(r.rankine, 2) == 0

    r = Degree(scale='ra', temp=0)
    assert r.scale == 'ra'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0
    assert round(r.rankine, 2) == 0


def test_propertties_setters():
    #
    # From 'ce' to 'fa'
    #
    r = Degree(scale='ce', temp=0)
    r.scale = 'fa'
    assert round(r.temp, 2) == 32.00
    assert round(r.celcius, 2) == 0
    assert round(r.fahrenheit, 2) == 32.00
    assert round(r.kelvin, 2) == 273.15
    assert round(r.rankine, 2) == 491.67
    #
    # From 'ce' to 'ke'
    #
    r = Degree(scale='ce', temp=0)
    r.scale = 'ke'
    assert round(r.temp, 2) == 273.15
    assert round(r.celcius, 2) == 0.00
    assert round(r.fahrenheit, 2) == 32.00
    assert round(r.kelvin, 2) == 273.15
    assert round(r.rankine, 2) == 491.67
    #
    # From 'ce' to 'ra'
    #
    r = Degree(scale='ce', temp=0)
    r.scale = 'ra'
    assert round(r.temp, 2) == 491.67
    assert round(r.celcius, 2) == 0.00
    assert round(r.fahrenheit, 2) == 32.00
    assert round(r.kelvin, 2) == 273.15
    assert round(r.rankine, 2) == 491.67
    #
    # From 'fa' to 'ce'
    #
    r = Degree(scale='fa', temp=0)
    r.scale = 'ce'
    assert round(r.temp, 2) == -17.78
    assert round(r.celcius, 2) == -17.78
    assert round(r.fahrenheit, 2) == 0.00
    assert round(r.kelvin, 2) == 255.37
    assert round(r.rankine, 2) == 459.67
    #
    # From 'fa' to 'ke'
    #
    r = Degree(scale='fa', temp=0)
    r.scale = 'ke'
    assert round(r.temp, 2) == 255.37
    assert round(r.celcius, 2) == -17.78
    assert round(r.fahrenheit, 2) == 0.00
    assert round(r.kelvin, 2) == 255.37
    assert round(r.rankine, 2) == 459.67
    #
    # From 'fa' to 'ra'
    #
    r = Degree(scale='fa', temp=0)
    r.scale = 'ra'
    assert round(r.temp, 2) == 459.67
    assert round(r.celcius, 2) == -17.78
    assert round(r.fahrenheit, 2) == 0.00
    assert round(r.kelvin, 2) == 255.37
    assert round(r.rankine, 2) == 459.67
    #
    # From 'ke' to 'ce'
    #
    r = Degree(scale='ke', temp=0)
    r.scale = 'ce'
    assert round(r.temp, 2) == -273.15
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00
    #
    # From 'ke' to 'fa'
    #
    r = Degree(scale='ke', temp=0)
    r.scale = 'fa'
    assert round(r.temp, 2) == -459.67
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00
    #
    # From 'ke' to 'ra'
    #
    r = Degree(scale='ke', temp=0)
    r.scale = 'ra'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00
    #
    # From 'ra' to 'ce'
    #
    r = Degree(scale='ra', temp=0)
    r.scale = 'ce'
    assert round(r.temp, 2) == -273.15
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00
    #
    # From 'ra' to 'fa'
    #
    r = Degree(scale='ra', temp=0)
    r.scale = 'fa'
    assert round(r.temp, 2) == -459.67
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00
    #
    # From 'ra' to 'ke'
    #
    r = Degree(scale='ra', temp=0)
    r.scale = 'ke'
    assert round(r.temp, 2) == 0.00
    assert round(r.celcius, 2) == -273.15
    assert round(r.fahrenheit, 2) == -459.67
    assert round(r.kelvin, 2) == 0.00
    assert round(r.rankine, 2) == 0.00

    with pytest.raises(Exception):
        r = Degree()
        r.scale = 'Dummy'


def test_add():
    r1 = Degree(scale='ce', temp=1.0)
    r2 = Degree(scale='ce', temp=20.0)
    r = r1 + r2
    assert r.scale == 'ce'
    assert r.temp == 21.00

    r1 = Degree(scale='fa', temp=1.0)
    r2 = Degree(scale='fa', temp=2.0)
    r = r1 + r2
    assert r.scale == 'fa'
    assert r.temp == 3.00

    r1 = Degree(scale='fa', temp=1.0)
    r = r1 + 2.00
    assert r.scale == 'fa'
    assert r.temp == 3.00

    r1 = Degree(scale='ke', temp=1.0)
    r = 2.00 + r1
    assert r.scale == 'ke'
    assert r.temp == 3.00

    r1 = Degree(scale='ce', temp=1.0)
    r2 = Degree(scale='ce', temp=1.0)
    r1 += r2
    assert r1.scale == 'ce'
    assert r1.temp == 2.00

    r1 = Degree(scale='ce', temp=0.0)
    r2 = Degree(scale='ke', temp=0.0)
    r1 += r2
    assert r1.scale == 'ce'
    assert r1.temp == -273.15

    r1 = Degree(scale='ra', temp=0.0)
    r2 = Degree(scale='ke', temp=0.0)
    r1 += r2
    assert r1.scale == 'ra'
    assert r1.temp == 0.00

    r1 = Degree(scale='ke', temp=0.0)
    r2 = Degree(scale='ra', temp=0.0)
    r1 += r2
    assert r1.scale == 'ke'
    assert r1.temp == 0.00


def test_sub():
    r1 = Degree(scale='ce', temp=1.0)
    r2 = Degree(scale='ce', temp=2.0)
    r = r1 - r2
    assert r.scale == 'ce'
    assert r.temp == -1.00

    r1 = Degree(scale='fa', temp=1.0)
    r = r1 - 2.00
    assert r.scale == 'fa'
    assert r.temp == -1.00

    r1 = Degree(scale='ce', temp=1.0)
    r2 = Degree(scale='ce', temp=1.0)
    r1 -= r2
    assert r1.scale == 'ce'
    assert r1.temp == 0.00

    r1 = Degree(scale='fa', temp=0.0)
    r2 = Degree(scale='ce', temp=0.0)
    r1 = r1 - r2
    assert r1.scale == 'fa'
    assert r1.temp == -32.00

    r1 = Degree(scale='ke', temp=0.0)
    r2 = Degree(scale='ra', temp=0.0)
    r1 = r1 - r2
    assert r1.scale == 'ke'
    assert r1.temp == 0.00

    r1 = Degree(scale='ra', temp=0.0)
    r2 = Degree(scale='ke', temp=0.0)
    r1 = r1 - r2
    assert r1.scale == 'ra'
    assert r1.temp == 0.00


def test_mul():
    r1 = Degree(scale='ce', temp=2.0)
    r = r1 * 10
    assert r.scale == 'ce'
    assert r.temp == 20.00

    r1 = Degree(scale='fa', temp=2.0)
    r = 20 * r1
    assert r.scale == 'fa'
    assert r.temp == 40.00

    r1 = Degree(scale='ke', temp=2.0)
    r = 20 * r1
    assert r.scale == 'ke'
    assert r.temp == 40.00

    r1 = Degree(scale='ra', temp=20.0)
    r = r1 * 2
    assert r.scale == 'ra'
    assert r.temp == 40.00


def test_div():
    r1 = Degree(scale='ce', temp=2.0)
    r = r1 / 10
    assert r.scale == 'ce'
    assert r.temp == 0.20

    r1 = Degree(scale='fa', temp=10.0)
    r = r1 / 10
    assert r.scale == 'fa'
    assert r.temp == 1.0

    r1 = Degree(scale='ke', temp=5.0)
    r = r1 / 2
    assert r.scale == 'ke'
    assert r.temp == 2.5

    r1 = Degree(scale='ra', temp=10.0)
    r = r1 / 2
    assert r.scale == 'ra'
    assert r.temp == 5.0


def test_str():
    r = Degree('ce')
    assert str(r) == '0.0 °C'

    r.scale = 'fa'
    r.temp = 0
    assert str(r) == '0.0 °F'

    r.scale = 'ke'
    r.temp = 0
    assert str(r) == '0.0 K'

    r.scale = 'ra'
    r.temp = 0
    assert str(r) == '0.0 °Ra'


def test_repr():
    r = Degree('ce')
    assert repr(r) == 'Degree(\'ce\', 0.0)'

    r.scale = 'fa'
    r.temp = 0
    assert repr(r) == 'Degree(\'fa\', 0.0)'

    r.scale = 'ke'
    r.temp = 0
    assert repr(r) == 'Degree(\'ke\', 0.0)'

    r.scale = 'ra'
    r.temp = 0
    assert repr(r) == 'Degree(\'ra\', 0.0)'



"""Temperatures conversions.

This module helps to create and manage Degree type objects.

Copyright (c) 2020 Thierry P.G. DECKER
All Rights Reserved.
Released under the MIT license

"""


class Degree:
    """Degree class.

    Exemples:

        Create Degree object:

            >>> Degree()
            Degree('ce', 0.0)
            >>> Degree(temp=10)
            Degree('ce', 10.0)
            >>> Degree(scale='fa')
            Degree('fa', 0.0)
            >>> Degree(scale='ke', temp=5)
            Degree('ke', 5.0)

        Additions:

            >>> Degree('fa', 10) + 1
            Degree('fa', 11.0)
            >>> r = Degree('fa', 10)
            >>> r += 1
            >>> r
            Degree('fa', 11.0)
            >>> 1 + Degree('ke', 10)
            Degree('ke', 11.0)
            >>> Degree('fa', 10) + Degree('fa', 10)
            Degree('fa', 20.0)
            >>> Degree('ce', 0) + Degree('fa', 32)
            Degree('ce', 0.0)

        Substractions (not commutative operation):

            >>> Degree('fa', 10) - 1
            Degree('fa', 9.0)
            >>> r = Degree('ke', 10)
            >>> r -= 1
            >>> r
            Degree('ke', 9.0)
            >>> Degree('ce', 0) - Degree('ke', 0)
            Degree('ce', 273.15)

        Multiplications:

            >>>Degree('ce', 10) * 2
            Degree('ce', 20.0)
            >>>3 * Degree('fa', 1)
            Degree('fa', 3.0)

        Divisions (not commutative operation):

            >>>Degree('ke', 10) / 2
            Degree('ce', 5.0)

        Representations:

            >>> str(Degree('fa', 1100.15))
            '1100.15 °F'
            >>> str(Degree('ke', 10.00))
            '10.00 K'
            >>> repr(Degree('ke', 100.15))
            "Degree('ke', 100.15)"

        Properties:

            >>> r = Degree('ce', 0)
            >>> r.celcius
            0.0
            >>> r.fahrenheit
            32.00
            >>> r = Degree('ce', 0)
            >>> r.scale = 'fa'
            >>> r.temp
            32.0

        Static methods:

            Temperature coversions:

                >>> Degree.conv_ce_to_fa(10)
                50.0
                >>> Degree.conv_ce_to_ke(0)
                273.15
                >>> Degree.conv_fa_to_ce(32)
                0.0
                >>> Degree.conv_fa_to_ke(32)
                273.15


    """
    _scales = {
        #
        # Supported scales
        #
        'ce',  # Celcius
        'fa',  # Fahrenheit
        'ke',  # Kelvin
        'ra'  # Rankine
    }

    def __init__(self, scale='ce', temp=0.):
        """Initialize new Degree instances.

        Raises:
            NameError if the given scale is not supported.
        """
        if scale not in self._scales:
            raise NameError(scale)
        self._scale = scale
        self._temp = float(temp)

    def __add__(self, other):
        new_scale = self.scale
        new_temp = self.temp
        if isinstance(other, Degree):
            if self.scale == 'ce':
                new_temp += other.celcius
            elif self.scale == 'fa':
                new_temp += other.fahrenheit
            elif self.scale == 'ke':
                new_temp += other.kelvin
            elif self.scale == 'ra':
                new_temp += other.rankine
        else:
            new_temp += float(other)
        return Degree(scale=new_scale, temp=new_temp)

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        new_scale = self.scale
        new_temp = self.temp
        if isinstance(other, Degree):
            if self.scale == 'ce':
                new_temp -= other.celcius
            elif self.scale == 'fa':
                new_temp -= other.fahrenheit
            elif self.scale == 'ke':
                new_temp -= other.kelvin
            elif self.scale == 'ra':
                new_temp -= other.rankine
        else:
            new_temp -= float(other)
        return Degree(scale=new_scale, temp=new_temp)

    def __mul__(self, other):
        return Degree(scale=self.scale, temp=self.temp * float(other))

    def __rmul__(self, other):
        return self.__mul__(other)

    def __truediv__(self, other):
        return Degree(scale=self.scale, temp=self.temp / float(other))

    def __str__(self):
        if self.scale == 'ce':
            return f'{self.temp} °C'
        if self.scale == 'fa':
            return f'{self.temp} °F'
        if self.scale == 'ke':
            return f'{self.temp} K'
        if self.scale == 'ra':
            return f'{self.temp} °Ra'

    def __repr__(self):
        return f'Degree(\'{self.scale}\', {self.temp})'

    @property
    def scales(self):
        """Supported temperature scales.

        Returns:
            A set of the managed scales.
        """
        return self._scales

    @property
    def scale(self):
        """Scale of the object.

        Returns:
            A string containing a scale included in the supported scales set.
        """
        return self._scale

    @scale.setter
    def scale(self, scale):
        """Set the scale

        Set the scale value and convert the temperature value in the new scale.

        Raises:
             NameError if the given scale is not supported.
        """
        if scale not in self._scales:
            raise NameError(scale)

        if scale != self._scale:

            if self._scale == 'ce' and scale == 'fa':
                self._temp = self.conv_ce_to_fa(temp=self._temp)
            if self._scale == 'ce' and scale == 'ke':
                self._temp = self.conv_ce_to_ke(temp=self._temp)
            if self._scale == 'ce' and scale == 'ra':
                self._temp = self.conv_ce_to_ra(temp=self._temp)

            if self._scale == 'fa' and scale == 'ce':
                self._temp = self.conv_fa_to_ce(temp=self._temp)
            if self._scale == 'fa' and scale == 'ke':
                self._temp = self.conv_fa_to_ke(temp=self._temp)
            if self._scale == 'fa' and scale == 'ra':
                self._temp = self.conv_fa_to_ra(temp=self._temp)

            if self._scale == 'ke' and scale == 'ce':
                self._temp = self.conv_ke_to_ce(temp=self._temp)
            if self._scale == 'ke' and scale == 'fa':
                self._temp = self.conv_ke_to_fa(temp=self._temp)
            if self._scale == 'ke' and scale == 'ra':
                self._temp = self.conv_ke_to_ra(temp=self._temp)

            if self._scale == 'ra' and scale == 'ce':
                self._temp = self.conv_ra_to_ce(temp=self._temp)
            if self._scale == 'ra' and scale == 'fa':
                self._temp = self.conv_ra_to_fa(temp=self._temp)
            if self._scale == 'ra' and scale == 'ke':
                self._temp = self.conv_ra_to_ke(temp=self._temp)

            self._scale = scale

    @property
    def temp(self):
        """Temperature of the object.

        The temperature is stored in the scale of the object.

        returns:
            A float containing the temperature of the object.
        """
        return self._temp

    @temp.setter
    def temp(self, temp):
        """Set the temperature

        """
        if float(temp) != self._temp:
            self._temp = float(temp)

    @property
    def celcius(self):
        """Celcius value.

        Returns:
            A float containg the Celcius value.
        """
        if self._scale == 'ce':
            return self._temp
        if self._scale == 'fa':
            return self.conv_fa_to_ce(self._temp)
        if self._scale == 'ke':
            return self.conv_ke_to_ce(self._temp)
        if self._scale == 'ra':
            return self.conv_ra_to_ce(self._temp)

    @property
    def fahrenheit(self):
        """Fahrenheit value.

        Returns:
            A float containing the Fahrenheit value.
        """
        if self._scale == 'ce':
            return self.conv_ce_to_fa(self._temp)
        if self._scale == 'fa':
            return self._temp
        if self._scale == 'ke':
            return self.conv_ke_to_fa(self._temp)
        if self._scale == 'ra':
            return self.conv_ra_to_fa(self._temp)

    @property
    def kelvin(self):
        """Kelvin value.

        Returns:
            A float containing the Kelvine value.
        """
        if self._scale == 'ce':
            return self.conv_ce_to_ke(self._temp)
        if self._scale == 'fa':
            return self.conv_fa_to_ke(self._temp)
        if self._scale == 'ke':
            return self._temp
        if self._scale == 'ra':
            return self.conv_ra_to_ke(self._temp)

    @property
    def rankine(self):
        """Fahrenheit value.

        Returns:
            A float containing the Fahrenheit value.
        """
        if self._scale == 'ce':
            return self.conv_ce_to_ra(self._temp)
        if self._scale == 'fa':
            return self.conv_fa_to_ra(self._temp)
        if self._scale == 'ke':
            return self.conv_ke_to_ra(self.temp)
        if self._scale == 'ra':
            return self._temp

    @staticmethod
    def conv_ce_to_fa(temp):
        """Convert Celcius value to Fahrenheit.

        Args:
            temp: A float containing the Celcius value to convert.

        Returns:
            A float containing the Fahrenheit value.
        """
        return ((9 * temp) / 5) + 32

    @staticmethod
    def conv_ce_to_ke(temp):
        """Convert Celcius value to Kelvin.

        Args:
            temp: A float containing the Celcius value to convert.

        Returns:
            A float containing the Kelvin value.
        """
        return temp + 273.15

    @staticmethod
    def conv_ce_to_ra(temp):
        """Convert Celcius value to Rankine.

        Args:
            temp: A float containing the Celcius value to convert.

        Returns:
            A float containing the Rankine value.
        """
        return temp * 1.8 + 32 + 459.67

    @staticmethod
    def conv_fa_to_ce(temp):
        """Convert Fahrenheit value to Celcius.

        Args:
            temp: A float containing the Fahrenheit value.

        returns:
            A Float containing the Celcius value.
        """
        return ((temp - 32) * 5) / 9

    @staticmethod
    def conv_fa_to_ke(temp):
        """Convert Fahrenheit value to Kelvin.

        Args:
            temp: A float containing the Fahrenheit value.

        returns:
            A Float containing the Kelvin value.
        """
        return ((temp + 459.67) * 5) / 9

    @staticmethod
    def conv_fa_to_ra(temp):
        """Convert Fahrenheit value to Rankine.

        Args:
            temp: A float containing the Fahrenheit value.

        returns:
            A Float containing the Rankine value.
        """
        return temp + 459.67

    @staticmethod
    def conv_ke_to_ce(temp):
        """Convert Kelvin value to Celcius.

        Args:
            temp: A float containing the Kelvin value.

        returns:
            A Float containing the Celcius value.
        """
        return temp - 273.15

    @staticmethod
    def conv_ke_to_fa(temp):
        """Convert Kelvin value to Fahrenheit.

        Args:
            temp: A float containing the Kelvin value.

        returns:
            A Float containing the Fahrenheit value.
        """
        return ((9 * temp) / 5) - 459.67

    @staticmethod
    def conv_ke_to_ra(temp):
        """Convert Kelvin value to Fahrenheit.

        Args:
            temp: A float containing the Kelvin value.

        returns:
            A Float containing the Rankine value.
        """
        return temp * 1.8

    @staticmethod
    def conv_ra_to_ce(temp):
        """Convert Rankine value to Celcius.

        Args:
            temp: A float containing the Rankine value.

        returns:
            A Float containing the Celcius value.
        """
        return (temp - 459.67 - 32) / 1.8

    @staticmethod
    def conv_ra_to_fa(temp):
        """Convert Rankine value to Fahrenheit.

        Args:
            temp: A float containing the Rankine value.

        returns:
            A Float containing the Fahrenheit value.
        """
        return temp - 459.67

    @staticmethod
    def conv_ra_to_ke(temp):
        """Convert Rankine value to Fahrenheit.

        Args:
            temp: A float containing the Rankine value.

        returns:
            A Float containing the Kelvin value.
        """
        return temp / 1.8



"""Disances conversions.

Copyright (c) 2020 Thierry P.G. DECKER
All Rights Reserved.
Released under the MIT license

"""


class Distance:
    """Distance class.

    """
    _scales = {
        'km',  # Kilometer
        'mi',  # Mile
    }

    def __init__(self, scale='km', dist=0.):
        """Initialize new Distance instances.

        Raises:
            NameError if the given scale is not supported.
        """
        if scale not in self._scales:
            raise NameError(scale)
        self._scale = scale
        self._dist = float(dist)

    def __add__(self, other):
        new_scale = self.scale
        new_dist = self.dist
        if isinstance(other, Distance):
            if self.scale == 'km':
                new_dist += other.kilometer
            elif self.scale == 'mi':
                new_dist += other.mile
        else:
            new_dist += float(other)
        return Distance(scale=new_scale, dist=new_dist)

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        new_scale = self.scale
        new_dist = self.dist
        if isinstance(other, Distance):
            if self.scale == 'km':
                new_dist -= other.kilometer
            elif self.scale == 'mi':
                new_dist -= other.mile
        else:
            new_dist -= float(other)
        return Distance(scale=new_scale, dist=new_dist)

    def __mul__(self, other):
        return Distance(scale=self.scale, dist=self.dist * float(other))

    def __rmul__(self, other):
        return self.__mul__(other)

    def __truediv__(self, other):
        return Distance(scale=self.scale, dist=self.dist / float(other))

    def __str__(self):
        if self.scale == 'km':
            return f'{self.dist} km'
        if self.scale == 'mi':
            return f'{self.dist} mi'

    def __repr__(self):
        return f'Distance(\'{self.scale}\', {self.dist})'

    @property
    def scales(self):
        """Supported distances scales.

        Returns:
            A set of the managed scales.
        """
        return self._scales

    @property
    def scale(self):
        """Scale of the object.

        Returns:
            A string containing a scale included in the supported scales set.
        """
        return self._scale

    @scale.setter
    def scale(self, scale):
        """Set the scale

        Set the scale value and convert the distance value in the new scale.

        Raises:
             NameError if the given scale is not supported.
        """
        if scale not in self._scales:
            raise NameError(scale)

        if scale != self._scale:

            if self._scale == 'km' and scale == 'mi':
                self.dist = self.conv_km_to_mi(dist=self.dist)
            if self._scale == 'mi' and scale == 'km':
                self.dist = self.conv_mi_to_km(dist=self.dist)

            self._scale = scale

    @property
    def dist(self):
        """Distance of the object.

        Returns:
            A Float containing a distance included in the object's scale.
        """
        return self._dist

    @dist.setter
    def dist(self, dist):
        """Set The distance

        """
        if float(dist) != self._dist:
            self._dist = float(dist)

    @property
    def kilometer(self):
        """Kilometer value.

        Returns:
            A float containg the Kilometer value.
        """
        if self._scale == 'km':
            return self._dist
        if self._scale == 'mi':
            return self.conv_mi_to_km(self._dist)

    @property
    def mile(self):
        """Mile value.

        Returns:
            A float containg the Mile value.
        """
        if self._scale == 'km':
            return self.conv_km_to_mi(self._dist)
        if self._scale == 'mi':
            return self._dist

    @staticmethod
    def conv_km_to_mi(dist):
        """Convert Kilometer value to Mile.

        Args:
            dist: A float containing the Kilometer value to convert.

        Returns:
            A float containing the Mile value.
        """
        return dist / 1.609

    @staticmethod
    def conv_mi_to_km(dist):
        """Convert Mile value to Kilometer.

        Args:
            dist: A float containing the Mile value to convert.

        Returns:
            A float containing the Kilomeer value.
        """
        return dist * 1.609



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
        name="siarnaq",
        version="1.0.0",
        author="Thierry P.G. DECKER",
        author_email="mail@thierry-decker.com",
        description="A conversion library",
        long_description=long_description,
        long_description_content_type="text/markdown",
        url="https://github.com/thierrydecker/siarnaq",
        packages=setuptools.find_packages(),
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        python_requires='>=3.8',
)
